// Author:  Temujin
//
// Based on: ai.cs by BDU Kcam
//
// Improvements over standard AI maps:
//   1 - Replaces standard (weak) vehicles with tougher unique configs (in the mis file)
//   2 - Replaces one pilot system with incremental approach.  Every kill leads to a tougher
//       replacement pilot as your enemy.
//   3 - Implemented AUTO-HEAL/RELOAD after each kill... for player AND AI
//   4 - Fixed multiple-enemy bugs.

$HEALVAL 	= 5000;
$AMMOVAL 	= 20;

exec("adv_ai_pilots.cs");

function setDefaultMissionOptions()
{
   $server::TeamPlay        = true;
   $server::AllowTeamPlay   = true;   
   $server::AllowDeathmatch = false;
   
   $server::AllowTeamYellow = true;
   $server::AllowTeamBlue   = true;
   $server::AllowTeamRed    = true;
   $server::AllowTeamPurple = false;

   $server::disableTeamYellow = false;
   $server::disableTeamBlue   = false;
   $server::disableTeamRed    = false;
   $server::disableTeamPurple = true;
}

// Vehicle Destruction...

function vehicle::onDestroyed( %victimVeh, %destroyerVeh )
{
   if( getTeam( %destroyerVeh ) != getTeam( %victimVeh ) )
   {
      %player = playerManager::vehicleIdToPlayerNum( %destroyerVeh );
      %player.numKills++;      
      $HumanKills++;
   }

   if( getTeam( %victimVeh ) == *IDSTR_TEAM_PURPLE )
   {
//    if you use resapawnVehicle instead of dropNewVehicle, you'll get the
//    same AI opponent back after you kill him.

      if( %victimVeh == %destroyerVeh)
         schedule( "respawnVehicle(" @ %victimVeh @ "," @ %destroyerVeh @ ");", 5 );

      %player = %victimVeh.owner;
	%player.AI = 0;
      schedule( "dropNewVehicle(" @ %player @ ");", 10 );

      %vehicleID = playerManager::PlayerNumToVehicleId(%player);
	if( %vehicleID == %destroyerVeh ) schedule("raiselevel("@ %player @");",2);
	schedule("deleteObject(" @ %victimVeh @ ");", 2);
   }
   else
   {
      %player = playerManager::vehicleIdToPlayerNum( %victimVeh );
	if( %player == %destroyerVeh.owner ) schedule("lowerlevel("@ %player @");",2);
      $AIKills++;
   }

   %message = getFancyDeathMessage(getHUDName(%victimVeh), getHUDName(%destroyerVeh));
   if(%message != "")
   {
      say( 0, 0, %message);
   }

   schedule("RnR("@%destroyerVeh@");", 1);
   schedule("RnR("@%destroyerVeh@");", 2);
   schedule("RnR("@%destroyerVeh@");", 3);
   schedule("RnR("@%destroyerVeh@");", 4);
   schedule("RnR("@%destroyerVeh@");", 5);
}

function respawnVehicle( %oldVeh, %destroyer )
{
   %player = playerManager::vehicleIdToPlayerNum( %destroyer );

   %x = getPosition(%destroyer,x)+250;
   %y = getPosition(%destroyer,y)+250;
   %z = getPosition(%destroyer,z)+250;

   %herc = hercVehicle(%oldVeh);
   setPosition(%herc, %x, %y, %z);
   schedule( "deleteObject(" @ %oldVeh @ ");", $respawnDealNoHq + 10 );
   addToSet( "MissionGroup", %herc );

   %herc.owner = %player;
   %player.AI = %herc;
}

function RnR( %vehicle )
{
   healObject(%vehicle, $HEALVAL);
   reloadObject(%vehicle, $AMMOVAL);
}

function raiselevel( %player )
{
   %player.level += 1;
   if( %player.level > 30 ) %player.level = 30;
   else 
   {
      %vehicleID = playerManager::PlayerNumToVehicleId(%player);
      say("everyone", 0, getHudName( %vehicleID ) @ " achieves level " @ %player.level @ "!", "alarm2.wav" );
   }
}

function lowerlevel( %player )
{
   %player.level -= 1;
   if( %player.level < 1 ) %player.level = 1;
   else
   {
	%vehicleID = playerManager::PlayerNumToVehicleId(%player);
	say("everyone", 0, getHudName( %vehicleID ) @ " falls to level " @ %player.level @ "!", "sfx_prob.wav" );
   }
}

// Vehicle Addition...

function vehicle::onAdd(%vehicle)
{
   %player = playerManager::vehicleIdToPlayerNum(%vehicle);

   if( %player ) // which means it's a human
   {
      echo ("--------- players AI ", %player.AI );

	if ( !%player.level ) schedule( "raiselevel( "@ %player @");", 2);   //Starting level at 1.

      if ( %player.AI ) 
         schedule( "order(" @ %player.AI @ ",attack," @ %vehicle @ ");", 5 );
      else
         schedule( "dropNewVehicle(" @ %player @ ");", 5 );
   }
}

function getRandomAI()
{
   // check for valid ranges
   // When building an adv_ai map, place 20 specially designed herc vehicles somewhere on the map (hidden)
   // name the objects as Missiongroup\\Enemy\\XX, where XX is a number from 1-20.  20 will be needed for
   // continued operations.

   %i = RandomInt(1,20);

   %vehiclepath = getObjectID("Missiongroup\\Enemy\\" @ %i );

   return cloneVehicle( %vehiclepath );
}

function dropNewVehicle( %player )
{
   if(!%player || %player.AI) return;

   %playerVeh = playerManager::playerNumToVehicleId(%player);                                                                                    
   %herc = getRandomAI();
   setTeam( %herc, *IDSTR_TEAM_PURPLE );

   %pilot = %player.level + 30;
   setPilotId(%herc, %pilot );

   // Vehicles above level 24 also have advanced R & R rates...
   if(%player.level > 24) schedule( "slowrnr(" @ %herc @ "," @ %player.level @ ");", 10);

   %x = getPosition(%playerVeh,x)+RandomInt(-750,750);
   %y = getPosition(%playerVeh,y)+RandomInt(-750,750);
   %z = getPosition(%playerVeh,z)+250;

   setPosition( %herc, %x, %y, %z);

   schedule( "order(" @ %herc @ ",attack," @ %playerVeh @ ");", 5 );

   %herc.owner = %player;
   %player.AI = %herc;
}

function slowrnr( %vehicle, %level )
{
   if(getHudName(%vehicle) != "unknown" && %level > 24)
   {
	%rate = (35 - %level);
      schedule( "slowrnr(" @ %vehicle @ "," @ %level @ ");", %rate);
   }
   healObject(%vehicle, 750);
   reloadObject(%vehicle, 8);
}
